#include "global.h"

//****************************************************
// global matrix of hamming codes (4,7)
//****************************************************
//0 15 19 28 37 42 54 57 70 73 85 90 99 108 112 127
const char HammingCode[16] = 
{
	0x00 , 0x0F , 0x13 , 0x1C ,
		0x25 , 0x2A , 0x36 , 0x39 ,
		0x46 , 0x49 , 0x55 , 0x5A ,
		0x63 , 0x6C , 0x70 , 0x7F 
};

//****************************************************
// global variables
// for descitptions see global.h
//****************************************************

char fifty_milli_second = false;
Time system_time = 0;

char command_waiting; //set to true if command waiting

unsigned char seq_nr = 0; //sequence number

unsigned char flash_LED = 0; 

char nr_of_slaves = 0;


unsigned char I2C_bfr_ptr=0; //set reception counter to 0
unsigned char I2C_error = false;
unsigned char I2C_block = true;
unsigned char I2C_nr_bytes = 0;
unsigned char I2C_in_buffer[I2C_BUFFER_SIZE] = {0};
unsigned char I2C_out_buffer[I2C_BUFFER_SIZE] = {0};

int ib_head = 0;
int ib_tail = 0;
char inputbuffer[ SERIAL_INPUTBUFFER_SIZE ];

int outbuf_head = 0;
int outbuf_tail = 0;
char serialout_buffer[ SERIAL_OUTPUTBUFFER_SIZE ];


/************************************************************************
 Pause routine
 ***********************************************************************/ 
void pause(long milliseconds)
{
  //remember TAR ticks with about 1usec
  Time endtime = system_time + TAR + 1000*milliseconds;
  Time ccurrenttime = system_time + TAR;
  char c;
  //just do nothing
  while (ccurrenttime <= endtime) 
    {
      ccurrenttime = system_time+TAR;
      c = P1IN;
    }
} 


//****************************************************
// Hamming Error Correction Functions
//****************************************************

char CorrectAndDecode(unsigned char code)
{
	unsigned char mask = 1;
	char n;
	char i;
	for (i = 0; i < 8; i++) 
	{
		n = A_CodeToData (code ^ mask);
		if (n != -1) 
			// Corrected it!
			return n;
		mask <<= 1;
	}
	// did not work
	return -1;
}

char A_CodeToData (unsigned char code)
{
  unsigned char i;
  for (i = 0; i < 16; i++)
    if (code == HammingCode[i])
      return (char) i;
  // Not a code!
  return -1;
}

char CodeToData(unsigned char code) //returns decoded (and corrected) Hamming code
{
  char n;
  n = A_CodeToData(code);
  if(n != -1)
    return n;
  n = CorrectAndDecode(code);
  return n;
}
