/**************************************************************************** 
*  global.h
*
*  Gergely Zaruba
*  July 2007
*
*****************************************************************************/
#ifndef __MY_GLOBAL_H__
#define __MY_GLOBAL_H__

#include <msp430x16x.h>
#include <stdlib.h>
#include <signal.h>
#include <stdio.h>
#include <string.h>

#include "serialout.h"


/************************************************************************
 General Macros
 ***********************************************************************/ 
#define VERBOSE_RESPONSE  //uncomment this line for comments flying back to the host

#ifndef false
#define false 0
#endif 

#ifndef true
#define true 1
#endif 

#ifndef FALSE
#define FALSE 0
#endif 

#ifndef TRUE
#define TRUE 1
#endif 


//****************************************************
// Macros - for communications protocol
//****************************************************
#define START_FRAME 0x81  //start frame delimiter is 129; one of only two 8 bit long words
#define END_FRAME 0xF1    //end frame delimiter is 241; the second of the 8 bit long words
#define RESPONSE_ERROR 'E'
#define RESPONSE_OK 'O'
#define RESPONSE_COMMENT 'C'
#define RESPONSE_VALUES 'V'

#define COMMAND_HELLO 'H'
#define COMMAND_GET_VALUE 'A'
#define COMMAND_GET_PERIODIC 'P'
#define COMMAND_PROG_SLAVES 'S'
#define COMMAND_TRIGGER_ADCS 'T'
#define COMMAND_SET_RATE 'R'
#define COMMAND_SET_NR_CONV 'N'
#define COMMAND_SET_SINGLE_RUN '1'
#define COMMAND_SET_CONTINOUS_RUN '9'
#define COMMAND_SET_SLAVE_RESET '0'

#define ERROR_TYPE_GENERAL 'Q'


//****************************************************
// Macros - for I2C communications 
//****************************************************
#define SLAVE_SET_NR_CONV 1
#define SLAVE_SET_RATE 2
#define SLAVE_SET_CONTINOUS 4
#define SLAVE_SET_SINGLE 8
#define SLAVE_SET_RESET 16


/************************************************************************
 Useful Macros
 ***********************************************************************/ 

#define SERIAL_OUTPUTBUFFER_SIZE 600 //the UART serial output buffer's size in bytes

#define SERIAL_INPUTBUFFER_SIZE 80 //the UART serial input buffer's size in bytes

#define MAX_ADC_SLAVES 20 //how many ADC slaves there can be (for indexing)

#define MAX_COMMAND_LENGTH 10 //maximum command length coming from host

#define I2C_BUFFER_SIZE 10 //buffer size for I2C transmissions

#define TRIGGER_CYCLES 20 //"clock cycles" to spend in trigger high mode
                          //for slave reading trigger
/************************************************************************
 Typedefs
 ***********************************************************************/ 
typedef long long Time;


//****************************************************
// global matrix of hamming codes (4,7)
//****************************************************
//0 15 19 28 37 42 54 57 70 73 85 90 99 108 112 127
extern const char HammingCode[16];

/************************************************************************
 Globals
 ***********************************************************************/ 
// will be set true every 50ms
// need to be set false by handler in main loop
extern char fifty_milli_second; 

// System time overflow variable; ticks with 0.244 msec
// needs to be used with TAR, i.e., system_time+TAR
extern Time system_time;

extern char command_waiting; //set to true if command waiting

extern unsigned char seq_nr; //sequence number

extern unsigned char flash_LED;  //flash register (turn bit one to make LED flash

extern char nr_of_slaves; //number of slaves attached to master

//I2C variables
extern unsigned char I2C_bfr_ptr;
extern unsigned char I2C_error;
extern unsigned char I2C_block;
extern unsigned char I2C_nr_bytes;
extern unsigned char I2C_in_buffer[I2C_BUFFER_SIZE];
extern unsigned char I2C_out_buffer[I2C_BUFFER_SIZE];

//UART serial in circular buffer and its pointers
extern int ib_head;
extern int ib_tail;
extern char inputbuffer[ SERIAL_INPUTBUFFER_SIZE ];

//UART serial out circular buffer and its pointers
extern int outbuf_head;
extern int outbuf_tail;
extern char serialout_buffer[ SERIAL_OUTPUTBUFFER_SIZE ];

/************************************************************************
 Forward Declarations (Prototypes)
 ***********************************************************************/ 
void pause(long); //wait
char A_CodeToData (unsigned char code); //for decoding (7,4) Hamming
char CodeToData (unsigned char code); //for decoding (7,4) Hamming
char CorrectAndDecode(unsigned char code); //for decoding (7,4) Hamming

#endif
