/**************************************************************************** 
*  serialin.c
*
*  Gergely Zaruba
*  August 2007
*
****************************************************************************/
#include "serialin.h"


/************************************************************************
 insert character received on UART0 to inputbuffer
 disable interrupts before function call 
 ***********************************************************************/ 
char critical serialin_insert_char( unsigned char c )
{
  if(c==END_FRAME)
    command_waiting=true;
  
  if ( (ib_head + 1 == ib_tail) || 
       (( ib_tail == 0) && (ib_head == SERIAL_INPUTBUFFER_SIZE-1 )) )
    return false;
  
  inputbuffer[ib_head] = c;
  ib_head++;
  ib_head %= SERIAL_INPUTBUFFER_SIZE;
  return true;
}

/************************************************************************
 remove head character received on UART1 to inputbuffer
 ***********************************************************************/ 
char critical serialin_get_char( char *c )
{
  // disable interrupts before function call 
  if ( ib_tail == ib_head)
    return false;
  
  *c = inputbuffer[ib_tail];  
  ib_tail++;
  ib_tail %= SERIAL_INPUTBUFFER_SIZE;
  return true;
}

/************************************************************************
 peek at head character received on UART1 to inputbuffer
 ***********************************************************************/ 
char critical serialin_peek_char( char *c )
{
  // disable interrupts before function call 
  if ( ib_tail == ib_head)
    return false;
  
  *c = inputbuffer[ib_tail];  
  return true;
}

/************************************************************************
 remove head character received on UART1 to inputbuffer
 ***********************************************************************/ 
char critical serialin_remove_head()
{
  if ( ib_tail == ib_head)
    return false;
  ib_tail++;
  ib_tail %= SERIAL_INPUTBUFFER_SIZE;
  return true;
}


/***************************************************************************
 return one character from the serial buffer, do not return until there is one
***************************************************************************/
char serialin_get_char_wait( char *c )
{
  Time Timeout = system_time + TAR + 1000; // 1000 milliseconds
  while ((system_time + TAR) < Timeout)
    {
        if (serialin_get_char(c))
	  return true;
    }
    return false;
}


/************************************************************************
 is the input buffer empty?
 ***********************************************************************/ 
char serialin_buffer_empty()
{
  return(ib_tail == ib_head);
}

/************************************************************************
 clear the input buffer
 ***********************************************************************/ 
void critical serialin_buffer_clear()
{
  ib_head = ib_tail = 0;
}

